@@ -10,7 +10,7 @@
  */
 
 #define PTHREAD_CANCEL_ASYNCHRONOUS 0
-#define PTHREAD_CANCEL_ENABLE 0
+#define PTHREAD_CANCEL_ENABLE NULL
 
 int pthread_setcanceltype(int type, int *oldtype);
 int pthread_setcancelstate(int state, int *oldstate);
@@ -18,4 +18,4 @@ int pthread_cancel(pthread_t thread_id);
 
 #endif // defined(__ANDROID__)
 
-#endif // defined(HAVE_PTHREAD)
+#endif // defined(HAVE_PTHREAD)
\ No newline at end of file
